<?php
if (!class_exists('Correo_Argentino_Shipping_Method')) {
    class Correo_Argentino_Shipping_Method extends WC_Shipping_Method
    {
        public function __construct()
        {
            $this->id = 'correoargentino';
            $this->method_title = __('Correo Argentino', 'correoargentino');
            $this->method_description = __('Configurá las credenciales de conexión a la API', 'correoargentino');
            $this->title = $this->settings['title'] ?? __('Correo Argentino Shipping', 'correoargentino');
            $this->init();
        }


        public function init()
        {
            try {
                global $ca_error;
                $this->init_settings();

                if (is_admin()) {
                    WC_Admin_Notices::remove_notice('ca_error');
                    $this->handleAdminForms();
                }

                add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
                add_action('admin_notices', array($this, 'display_errors', $ca_error));
            } catch (Exception $e) {
                WC_Admin_Settings::add_error('Error: ' . $e->getMessage());
            }
        }

        private function handleAdminForms()
        {
            $isConnected = Utils::isConnected();
            $currentServiceType = Utils::getCurrentServiceType();

            $this->handleServiceSelectorForm($currentServiceType, $isConnected);
            $this->handleUserValidateMiCorreoForm();
            $this->handleLoginPaqArForm();
            $this->handleRegisterMiCorreoForm();
            $this->handleBusinessForm($isConnected);
        }

        private function handleServiceSelectorForm($currentServiceType, $isConnected)
        {
            if (isset($_GET['page'], $_GET['section'], $_GET['form']) && $_GET['page'] === 'wc-settings' && $_GET['section'] === 'correoargentino' && $_GET['form'] == 'service-selector') {
                if (isset($currentServiceType) && $isConnected) {
                    $formType = $currentServiceType == PAQ_AR ? CA_LOGIN_PAQ_AR_FORM : CA_USER_VALIDATE_MI_CORREO_FORM;
                    wp_safe_redirect('?page=wc-settings&tab=shipping&section=correoargentino&form=' . $formType);
                    return;
                }
                $this->method_description = __('Ahora seleccioná con cuál servicio querés operar.', 'correoargentino');
                $this->form_fields = CorreoArgentinoAdminForm::getServiceSelectorForm();
            }
        }

        private function handleUserValidateMiCorreoForm()
        {
            $is_valid_request = isset($_GET['page'], $_GET['section'], $_GET['form'])
                && $_GET['page'] === 'wc-settings'
                && $_GET['section'] === CA_PLUGIN_ID
                && $_GET['form'] == CA_USER_VALIDATE_MI_CORREO_FORM;

            if ($is_valid_request) {
                if (CorreoArgentinoMiCorreoService::isValidToken()) {
                    WC_Admin_Settings::add_message('Ya podés operar en MiCorreo, actualizá estos datos solo si deseas renovar la sesión.');
                }
                $this->form_fields = CorreoArgentinoAdminForm::getUserValidateMiCorreoForm();
            }
        }


        private function handleLoginPaqArForm()
        {
            $is_valid_request = isset($_GET['page'], $_GET['section'], $_GET['form'])
                && $_GET['page'] === 'wc-settings'
                && $_GET['section'] === CA_PLUGIN_ID
                && $_GET['form'] == CA_LOGIN_PAQ_AR_FORM;

            if ($is_valid_request) {
                $this->form_fields = CorreoArgentinoAdminForm::getLoginPaqArForm();
            }
        }


        private function handleRegisterMiCorreoForm()
        {
            $is_valid_request = isset($_GET['page'], $_GET['section'], $_GET['form'])
                && $_GET['page'] === 'wc-settings'
                && $_GET['section'] === CA_PLUGIN_ID
                && $_GET['form'] == CA_BUSINESS_MI_CORREO_FORM;

            if ($is_valid_request) {
                $this->method_description = __('Ahora agregá los datos de tu negocio.', 'correoargentino');
                $this->form_fields = CorreoArgentinoAdminForm::getMiCorreoBusinessForm();
            }
        }


        private function handleBusinessForm($isConnected)
        {
            $is_valid_request = isset($_GET['page'], $_GET['section'], $_GET['form'])
                && $_GET['page'] === 'wc-settings'
                && $_GET['section'] === CA_PLUGIN_ID
                && $_GET['form'] == 'business';

            if ($is_valid_request) {
                if (!$isConnected) {
                    wp_safe_redirect('?page=wc-settings&tab=shipping&section=correoargentino');
                    exit;
                }
                $this->method_description = __('Ahora agregá los datos de tu negocio.', 'correoargentino');
                $this->form_fields = CorreoArgentinoAdminForm::getBusinessForm();
            }
        }


        /**
         * @throws Exception
         */
        public function get_rates()
        {
            if ($this->shouldSkip()) {
                return;
            }

            $postalCode = $this->updateDestinationPostcode();
            $product = $this->getMostValuedProductInCart();
            $dimensions = $this->getProductDimensions($product);

            $service = (new CorreoArgentinoServiceFactory())->get();

            return $this->fetchRates($service, $postalCode, $dimensions);
        }

        /**
         * @return bool
         */
        private function shouldSkip()
        {
            return is_admin() && !defined('DOING_AJAX') && !is_cart() && !is_checkout();
        }

        private function updateDestinationPostcode()
        {
            $cart = WC()->cart;
            $package = $cart->get_shipping_packages();

            if (is_checkout() && Utils::getUseRates() && WC()->session->get(CA_CHOSEN_BRANCH_ZIP_CODE) !== null) {
                $package[0]['destination']['postcode'] = Utils::normalizeZipCode(WC()->session->get(CA_CHOSEN_BRANCH_ZIP_CODE));
            }

            return $package[0]['destination']['postcode'];
        }

        private function getMostValuedProductInCart()
        {
            $cart = WC()->cart;
            $packages = $cart->get_shipping_packages();
            $maxVolumetricWeight = 0;
            $mostValuedProduct = null;

            foreach ($packages as $package) {
                foreach ($package['contents'] as $item) {
                    $productId = $item['product_id'];
                    $product = wc_get_product($productId);

                    $dimensions = $this->getProductDimensions($product);
                    $volumetricWeight = ($dimensions["length"] * $dimensions["width"] * $dimensions["height"]);

                    if ($volumetricWeight > $maxVolumetricWeight) {
                        $maxVolumetricWeight = $volumetricWeight;
                        $mostValuedProduct = $product;
                    }
                }
            }

            return $mostValuedProduct;
        }


        private function getProductDimensions($product): array
        {
            return [
                "weight" => Utils::fromKgToGrams($product->get_weight()),
                "height" => floatval($product->get_height()),
                "width" => floatval($product->get_width()),
                "length" => floatval($product->get_length())
            ];
        }

        private function validateProductDimensions(): bool
        {
            $cart = WC()->cart;
            $packages = $cart->get_shipping_packages();
            $maxWeight = CA_MAX_WEIGHT;
            $maxLength = CA_MAX_LENGTH;

            foreach ($packages as $package) {
                foreach ($package['contents'] as $item) {
                    $productId = $item['product_id'];
                    $product = wc_get_product($productId);
                    $dimensions = $this->getProductDimensions($product);

                    if ($dimensions["weight"] > $maxWeight || $dimensions["length"] > $maxLength) {
                        return false;
                    }
                }
            }

            return true;
        }

        private function fetchRates($service, $postalCode, $dimensions)
        {
            $rateList = [];
            if (!Utils::canListRates()) {
                return;
            }

            $ratesHome = $service->getRates($postalCode, 'D', $dimensions);

            // @todo: this disables Correo Argentino Hoy - Domicilio
            $ratesHomeFiltered = array_filter($ratesHome["rates"], function ($rate) {
                return $rate['productName'] !== "Correo Argentino Hoy";
            });
            $ratesHome["rates"] = $ratesHomeFiltered;

            $ratesAgency = $service->getRates($postalCode, 'S', $dimensions);

            if (!isset($ratesHome["rates"]) || !isset($ratesAgency["rates"])) {
                return $rateList;
            }

            $rates = [
                "rates" => array_merge($ratesHome["rates"], $ratesAgency["rates"])
            ];

            if (!empty($rates['rates'])) {
                foreach ($rates['rates'] as $i => $rate) {
                    if ($rate && $rate['productName']) {
                        $branchLabel = $rate['deliveredType'] === 'S' ? '_branch' : '';
                        $rateList[] = [
                            "ID" => md5($rate['productName'] . $rate['deliveredType']) . $branchLabel,
                            "serviceName" => $rate['productName'],
                            "description" => $rate['productName'],
                            "serviceCode" => $rate['productType'],
                            'deliveredType' => $rate['deliveredType'],
                            "currency" => "ARS",
                            "totalPrice" => (float)$rate['price']
                        ];
                    }
                }
            }

            return $rateList;
        }

        private function getElementById($array, $id)
        {
            foreach ($array as $index => $element) {
                if (isset($element['ID']) && $element['ID'] == $id) {
                    return ['element' => $element, 'index' => $index];
                }
            }
            return null;
        }


        public function calculate_shipping($package = array())
        {
            global $count;
            $v = microtime();
            $rates = $this->get_rates();
            $data = ["rates" => $rates];
            $deliveryTypes = ["S" => BRANCH, "D" => HOME];

            if (!Utils::canListRates()) {
                return;
            }

            if (!$this->validateProductDimensions()) {
                if ($count < 1) {
                    Utils::message('Uno o mas paquetes exceden el peso y/o medidas máximas permitidas por Correo Argentino', 'error', false, false);
                }
                $count++;
                return;
            }

            if (!isset($data['rates'])) {
                return;
            }

            $chosenShippingMethod = WC()->session->get('chosen_shipping_methods');
            $chosenShippingMethodValue = $chosenShippingMethod[0];

            if (is_checkout() && !empty($chosenShippingMethodValue)) {
                $newRate = $this->getElementById($rates, $chosenShippingMethodValue);
                $data['rates'] = [];
                $index = $newRate['index'];
                $data['rates'][$index] = $newRate['element'];
            }

            foreach ($data['rates'] as $i => $rate) {
                $isBranch = $rate['deliveredType'] === 'S' ? 1 : 0;
                $branchLabel = $isBranch ? '_branch' : '';
                if (!empty($rate)) {
                    $this->add_rate(
                        array(
                            'id' => md5($rate['description'] . $rate['deliveredType']) . $branchLabel,
                            'label' => $rate['description'] . " - " . $deliveryTypes[$rate['deliveredType']],
                            'cost' => (float)$this->displayPrice($rate['totalPrice']),
                            'calc_tax' => 'per_order',
                            'version' => $v,
                            'meta_data' => array(
                                CA_IS_BRANCH => $isBranch,
                                CA_CHOSEN_SERVICE_TYPE => Utils::getCurrentServiceType(),
                                CA_CHOSEN_PRODUCT_TYPE => $rate['serviceCode'],
                                CA_CHOSEN_BRANCH => null,
                                CA_CHOSEN_BRANCH_NAME => null,
                                CA_CHOSEN_BRANCH_ZIP_CODE => null
                            )
                        )
                    );
                }
            }


        }

        private function displayPrice($price)
        {
            return Utils::getUseRates() && is_checkout() ? $price : 0;
        }
    }
}