<?php
defined('ABSPATH') || exit;

add_action('wp_enqueue_scripts', 'woo_dequeue_select2', 100);
add_action('woocommerce_review_order_before_cart_contents', 'correoargentino_validate_order', 10);
add_action('woocommerce_after_checkout_validation', 'correoargentino_validate_order', 10);
add_filter('woocommerce_shipping_methods', 'add_Correo_Argentino_Shipping_Method');
add_action('woocommerce_shipping_init', 'Correo_Argentino_Shipping_Method');
add_action('add_meta_boxes', [CorreoArgentinoMetabox::class, 'create']);
add_action('rest_api_init', [CorreoArgentinoLabel::class, 'create']);
add_action('admin_menu', [CorreoArgentinoOrder::class, 'build_admin_menu']);
add_action('woocommerce_after_shipping_rate', [CorreoArgentinoBranch::class, 'showBranches']);
add_action('woocommerce_review_order_before_payment', 'branch_info_handler');
add_action('wp_ajax_update_branch_handler', 'update_branch_handler');
add_action('wp_ajax_nopriv_update_branch_handler', 'update_branch_handler');
add_action('wp_ajax_update_province_handler', 'update_province_handler');
add_action('wp_ajax_nopriv_update_province_handler', 'update_province_handler');
add_action('wp_ajax_process_orders_handler', 'process_orders_handler');
add_action('wp_ajax_nopriv_process_orders_handler', 'process_orders_handler');
add_action("wp_enqueue_scripts", "add_select2_scripts");
add_action('wp_footer', 'choose_branch_handler');
add_action('wp_footer', 'enqueue_global_scripts');
add_action('woocommerce_new_order', 'new_order_handler');
//add_action('woocommerce_order_status_completed', 'on_payment_complete_handler', 10);
add_action('woocommerce_order_status_cancelled', 'on_order_cancelled_handler', 10);
add_action('woocommerce_update_options_shipping_' . CA_PLUGIN_ID, 'on_settings_updated_handler');

add_action('admin_enqueue_scripts', 'enqueue_admin_scripts');

// Keep this to ensure the request works properly
add_filter('http_request_timeout', function ($timeout) {
    return 15;
});

function __set_curl_to_follow(&$handle)
{
    curl_setopt($handle, CURLOPT_FOLLOWLOCATION, true);
}

add_action('http_api_curl', '__set_curl_to_follow');

// Billing and shipping addresses fields
add_filter('woocommerce_default_address_fields', 'filter_default_address_fields', 20, 1);
function filter_default_address_fields($address_fields)
{
    unset($address_fields['address_2']);
    $address_fields['address_2']['required'] = true;
    $address_fields['address_2']['priority'] = 60;
    $address_fields['address_2']['placeholder'] = STREET_NUMBER;
    $address_fields['address_2']['label'] = STREET_NUMBER;

    return $address_fields;
}

add_filter('woocommerce_billing_fields', 'filter_billing_fields', 20, 1);
function filter_billing_fields($billing_fields)
{
    $billing_fields['billing_address_2']['required'] = true;
    $billing_fields['billing_address_2']['placeholder'] = STREET_NUMBER;
    $billing_fields['billing_address_2']['label'] = STREET_NUMBER;
    return $billing_fields;
}

function varnish_safe_http_headers()
{
    header('X-UA-Compatible: IE=edge,chrome=1');
    session_cache_limiter('');
    header("Cache-Control: public, s-maxage=120");
    if (!session_id()) {
        session_start();
    }
}

add_action('send_headers', 'varnish_safe_http_headers');

add_filter('woocommerce_before_checkout', 'clear_wc_shipping_rates_cache');

function clear_wc_shipping_rates_cache()
{
    $packages = WC()->cart->get_shipping_packages();

    foreach ($packages as $key => $value) {
        $shipping_session = "shipping_for_package_$key";

        unset(WC()->session->$shipping_session);
    }
}

function enforce_update_shipping_methods()
{
    if (is_cart()) {
        WC()->cart->calculate_shipping();
        WC()->cart->calculate_totals();
    }
}

add_action('woocommerce_before_cart', 'enforce_update_shipping_methods');

function custom_admin_styles()
{
    echo '<style>
        .wp-list-table .column-column_cb {
            width: 18px; 
        }
    </style>';
}

add_action('admin_head', 'custom_admin_styles');

function process_bulk_orders_handler()
{
    if (Utils::getCurrentServiceType() === MI_CORREO) {
        ?>
        <script type="text/javascript">
            jQuery(document).ready(function ($) {
                const mainCheckboxes = $('.wc-correoargentino-all-orders');
                const orderCheckboxes = $('input[name="order_id[]"]');

                mainCheckboxes.on('change', function () {
                    const isChecked = $(this).prop('checked');
                    mainCheckboxes.prop('checked', isChecked);
                    orderCheckboxes.prop('checked', isChecked);
                });

                orderCheckboxes.on('change', function () {
                    const allChecked = orderCheckboxes.filter(':checked').length === orderCheckboxes.length;
                    mainCheckboxes.prop('checked', allChecked);
                });


                $('.bulk-actions-button').on('click', function () {
                    $(this).attr('disabled', true);
                    console.log('Processing...');
                    const selectedAction = $('.bulk-actions-top option:selected').val();

                    if (selectedAction == -1) return

                    const selectedOrderIds = [];


                    $('input[name="order_id[]"]:checked').each(function () {
                        selectedOrderIds.push($(this).val());
                    });

                    if (selectedOrderIds.length > 10) {
                        alert('Solo se permiten 10 registros por operación');
                        return;
                    }

                    const path = "<?php echo admin_url('admin-ajax.php'); ?>";

                    const promises = selectedOrderIds.map(orderId => {
                        showLoadingMessage(orderId);
                        return new Promise((resolve, reject) => {
                            $.ajax({
                                type: 'POST',
                                url: path,
                                data: {
                                    action: 'process_orders_handler',
                                    order_id: orderId,
                                },
                                success: function (response) {
                                    resolve(response);
                                },
                                error: function (error) {
                                    reject(error);
                                },
                                complete: function () {
                                    hideLoadingMessage(orderId);
                                }
                            });
                        });
                    });

                    Promise.all(promises)
                        .then(results => {
                            results.forEach(result => {
                                const {order_id, reference, api_error_message} = result.data;
                                if (!reference) {
                                    handleError(order_id, 'Error: ' + api_error_message);
                                    return
                                }
                                handleSuccess(order_id, reference);
                            });
                            
                            new Promise(resolve => setTimeout(resolve, 3000)).then(() => {
                                window.location.reload();
                            });
                        }).catch(errors => {
                            console.error('Error in processing orders:', errors);
                            errors.forEach(error => {
                                console.log(error)
                            });
                        });

                    function handleSuccess(orderId, reference) {
                        const trackingColumn = $('.ph-tracking_number-' + orderId);
                        const url = reference;
                        // Find the checkbox in the same table row and hide it
                        const checkbox = trackingColumn.closest('tr').find('input[type="checkbox"]');
                        checkbox.hide();
                        trackingColumn.empty();
                        trackingColumn.html(url);
                    }

                    function handleError(orderId, status) {
                        const statusColumn = $('.ph-tracking_number-' + orderId);   
                        statusColumn.html('<span style="color: #ff645c;">' + status + '</span>');
                    }

                    function showLoadingMessage(orderId) {
                        $('.ph-tracking_number-' + orderId).text('Procesando...');
                    }

                    function hideLoadingMessage(orderId) {
                        $('.ph-tracking_number-' + orderId).text('');
                    }
                });
            });
        </script>
        <?php
    }
}

add_action('admin_footer', 'process_bulk_orders_handler');

add_action('admin_init', function () {
    if (empty(Utils::getCurrentServiceType()) || !Utils::isConnected() || isset($_GET['form']) && $_GET['form'] === CA_BUSINESS_DETAILS_FORM) {
        return;
    }
    if (!Utils::canListRates()) {
        Utils::message(CA_BUSINESS_DETAILS_NOT_FILLED_MESSAGE, 'warning');
    }
});

add_filter('woocommerce_checkout_get_value', 'custom_edit_shipping_default_values', 10, 2);

function custom_edit_shipping_default_values($value, $input)
{
    $chosenShippingMethod = WC()->session->get('chosen_shipping_methods');
    $isBranch = Utils::isBranch($chosenShippingMethod[0]);
    $fields_to_modify = array(
        'shipping_city',
        'shipping_state',
        'shipping_postcode',
        'shipping_company',
        'shipping_address_1',
        'shipping_address_2'
    );

    if (!$isBranch) {
        WC()->session->__unset(CA_CHOSEN_BRANCH);
        if (in_array($input, $fields_to_modify) && $input === 'shipping_company') {
            $value = '';
        }
        return $value;
    }

    if (in_array($input, $fields_to_modify)) {
        switch ($input) {
            case 'shipping_postcode':
                // PostalCode
                $value = WC()->session->get(CA_CHOSEN_BRANCH_ZIP_CODE);
                break;
            case 'shipping_city':
                // City
                $value = WC()->session->get(CA_CHOSEN_BRANCH_CITY);
                break;
            case 'shipping_state':
                // State
                $value = WC()->session->get(CA_CHOSEN_PROVINCE_CODE);
                break;
            case 'shipping_address_1':
                // StreetName
                $value = WC()->session->get(CA_CHOSEN_BRANCH_STREET_NAME) . ' (' . WC()->session->get(CA_CHOSEN_BRANCH) . ')';
                break;
            case 'shipping_address_2':
                // StreetNumber
                $value = WC()->session->get(CA_CHOSEN_BRANCH_STREET_NUMBER);
                break;
            case 'shipping_company':
                // AgencyName
                $value = WC()->session->get(CA_CHOSEN_BRANCH_NAME);
                break;
        }
    }

    return $value;
}

add_action('woocommerce_thankyou', 'flush_chosen_branch_data');

function flush_chosen_branch_data($order_id)
{
    if (is_wc_endpoint_url('order-received')) {
        unset_wc_chosen_branch_values();
        restore_wc_shipping_address($order_id);
    }
}

function update_wc_customer_shipping_address($customer_id)
{
    update_user_meta($customer_id, 'shipping_address_1', WC()->session->get(CA_CUSTOMER_SHIPPING_ADDRESS_1));
    update_user_meta($customer_id, 'shipping_address_2', WC()->session->get(CA_CUSTOMER_SHIPPING_ADDRESS_2));
    update_user_meta($customer_id, 'shipping_city', WC()->session->get(CA_CUSTOMER_SHIPPING_CITY));
    update_user_meta($customer_id, 'shipping_state', WC()->session->get(CA_CUSTOMER_SHIPPING_STATE));
    update_user_meta($customer_id, 'shipping_postcode', WC()->session->get(CA_CUSTOMER_SHIPPING_POSTCODE));
    update_user_meta($customer_id, 'shipping_company', WC()->session->get(CA_CUSTOMER_SHIPPING_COMPANY));

    wp_update_user(array('ID' => $customer_id));
}

function restore_wc_shipping_address($order_id)
{
    $order = wc_get_order($order_id);
    $customer_id = $order->get_customer_id();

    update_wc_customer_shipping_address($customer_id);
}

function unset_wc_chosen_branch_values()
{
    WC()->session->__unset(CA_CHOSEN_BRANCH_ZIP_CODE);
    WC()->session->__unset(CA_CHOSEN_BRANCH_CITY);
    WC()->session->__unset(CA_CHOSEN_PROVINCE_CODE);
    WC()->session->__unset(CA_CHOSEN_BRANCH_STREET_NAME);
    WC()->session->__unset(CA_CHOSEN_BRANCH);
    WC()->session->__unset(CA_CHOSEN_BRANCH_NAME);
    WC()->session->__unset(CA_CHOSEN_BRANCH_STREET_NUMBER);
}

function set_temp_shipping_address($data)
{
    $user = wp_get_current_user();

    $shipping_address = array(
        'address_1' => get_user_meta($user->ID, 'shipping_address_1', true),
        'address_2' => get_user_meta($user->ID, 'shipping_address_2', true),
        'city' => get_user_meta($user->ID, 'shipping_city', true),
        'state' => get_user_meta($user->ID, 'shipping_state', true),
        'postcode' => get_user_meta($user->ID, 'shipping_postcode', true),
        'company' => get_user_meta($user->ID, 'shipping_company', true),
    );

    WC()->session->set(CA_CUSTOMER_SHIPPING_ADDRESS_1, $shipping_address['address_1']);
    WC()->session->set(CA_CUSTOMER_SHIPPING_ADDRESS_2, $shipping_address['address_2']);
    WC()->session->set(CA_CUSTOMER_SHIPPING_CITY, $shipping_address['city']);
    WC()->session->set(CA_CUSTOMER_SHIPPING_STATE, $shipping_address['state']);
    WC()->session->set(CA_CUSTOMER_SHIPPING_POSTCODE, $shipping_address['postcode']);
    WC()->session->set(CA_CUSTOMER_SHIPPING_COMPANY, $shipping_address['company']);

}

add_action('woocommerce_before_cart', 'set_temp_shipping_address');

add_filter('woocommerce_cart_shipping_packages', 'restore_initial_shipping_address');

function restore_initial_shipping_address($packages)
{
    $user = wp_get_current_user();
    $customer_id = (string)$user->ID;

    if (!empty($packages) && is_cart()) {
        update_wc_customer_shipping_address($customer_id);
    }

    return $packages;
}

add_action('woocommerce_customer_save_address', 'save_wc_customer_shipping_address', 10, 2);

function save_wc_customer_shipping_address($customer_id, $load_address)
{
    if ($load_address === 'shipping') {
        set_temp_shipping_address($customer_id);
    }
}


function ts_hide_ship_to_different_address_checkbox() {
    $chosenShippingMethod = WC()->session->get('chosen_shipping_methods');
    $isBranch = Utils::isBranch($chosenShippingMethod[0]);
    if (is_checkout() && $isBranch) {
        echo '<style>#ship-to-different-address label { visibility: hidden; } .shipping_address { visibility: hidden !important; display: none !important; }</style>';
    }
}
add_action('wp_head', 'ts_hide_ship_to_different_address_checkbox');

add_action('admin_enqueue_scripts', 'enqueue_woocommerce_admin_styles');

function enqueue_woocommerce_admin_styles() {
    // Check if WooCommerce is active
    if (class_exists('WooCommerce')) {
        // Enqueue WooCommerce admin stylesheets
        wp_enqueue_style('woocommerce_admin_styles', plugins_url('woocommerce/assets/css/admin.css'));
    }
}

add_filter('woocommerce_get_cart_url', 'custom_cart_page_url');
function custom_cart_page_url($url) {
    $new_cart_page_slug = 'cart-2';

    $new_cart_page_id = url_to_postid(home_url('/') . $new_cart_page_slug);

    if ($new_cart_page_id) {
        $new_cart_page_permalink = get_permalink($new_cart_page_id);

        if ($new_cart_page_permalink) {
            $url = $new_cart_page_permalink;
        }
    }

    return $url;
}

