<?php

class Utils
{
    const REQUIRED_SIGN = '<sup>*</sup>';

    static function getProvinces(): array
    {
        $provinces = json_decode(file_get_contents(plugin_dir_path(__FILE__) . '../Mock/provinces.json'), true);
        $results = [];
        foreach ($provinces["provincias"] as $item) {
            $code = $item['iso_id'];
            $results[$code] = $item['nombre'];
        }
        asort($results);
        return $results;

    }

    static function getDocumentTypes(): array
    {
        return array("DNI" => "DNI", "CUIT" => "CUIT");
    }

    static function getAvailableServices(): array
    {
        return array(
            "miCorreo" => "PAQ.AR API MiCorreo",
            "paq.ar" => "PAQ.AR API 2.0"
        );
    }

    /**
     * @param $code
     * @return array |string|null
     */
    public static function normalizeZipCode($code)
    {
        return preg_replace("/[^0-9]/", "", $code);
    }

    public static function cleanPhone($phone)
    {
        return preg_replace("/\-/", "", $phone);
    }


    public static function getCurrentServiceType()
    {
        return get_option(CA_SERVICE_TYPE);
    }

    public static function getUseRates()
    {
        return get_option(CA_USE_RATES);
    }

    public static function isConnected()
    {
        return get_option(CA_IS_CONNECTED);
    }

    public static function canListRates()
    {
        $settings = self::getSettings();

        $requiredFields = array(
            'business_name',
            'state',
            'city_name',
            'street_name',
            'street_number',
            'zip_code',
        );

        foreach ($requiredFields as $field) {
            if (empty($settings[$field])) {
                return false;
            }
        }

        return true;

    }

    public static function getSettings()
    {
        return maybe_unserialize(get_option(CA_SETTINGS));
    }

    public static function businessDetailsFilled()
    {
        return get_option(CA_BUSINESS_DETAILS_FILLED);
    }

    public static function getCredentials()
    {
        return maybe_unserialize(get_option(CA_CREDENTIALS));
    }

    public static function trace($v, $die = true, $type = 'notice')
    {
        echo '<div class="' . $type . ' inline"><pre>';
        var_dump($v);
        echo '</pre></div>';
        if ($die)
            die();
    }

    public static function shippingAddressBox($branchName, $branchId, $branchZipCode, $branchCity, $branchState, $branchStreetName, $branchStreetNumber, $cartUrl)
    {
        ?>
        <table class="shop_table">
            <thead>
            <tr>
                <th>Tu pedido será enviado a la siguiente sucursal:</th>
            </tr>
            </thead>
            <tbody>
            <tr>
                <td>
                    <address>
                        <?php echo esc_html($branchName) . " (" . esc_html($branchId) . '),<br>' . esc_html($branchStreetName) . ' ' . esc_html($branchStreetNumber) . ',<br>' . esc_html($branchCity) . ', ' . esc_html($branchState) . ', ' . esc_html($branchZipCode); ?>
                    </address>
                </td>
            </tr>
            </tbody>
            <tfoot>
            <tr>
                <td><a href="<?php echo esc_url($cartUrl); ?>">Cambiar sucursal</a></td>
            </tr>
            </tfoot>
        </table>
        <?php
    }

    public static function message($message, $type = 'message', $dismissible = false, $attach = true)
    {
        $classes = 'notice notice-' . esc_attr($type);
        if (!is_admin()) {
            $classes .= ' woocommerce-' . esc_attr($type);
        }
        if ($dismissible) {
            $classes .= ' is-dismissible';
        }
        if ($attach) {
            add_action('admin_notices', function () use ($message, $classes) {
                echo '<div class="' . esc_attr($classes) . '"><p>' . $message . '</p></div>';
            });
        } else {
            echo '<div class="' . esc_attr($classes) . '"><p>' . $message . '</p></div>';
        }
    }

    public static function isBranch($chosenShippingMethod)
    {
        // Check if the string ends with '_branch'
        return str_ends_with($chosenShippingMethod, '_branch');
    }

    public static function fromKgToGrams($kg)
    {
        // Remove thousand separators (,)
        $kg = str_replace(',', '', $kg);

        // Replace the last dot (.) with an empty string if multiple dots are present
        $kg = preg_replace('/\.(\d*)\./', '.$1', $kg);

        // Replace the first dot (.) with an empty string if at the beginning of the number
        $kg = preg_replace('/^(\.)/', '', $kg);

        // Convert the input to a float and then multiply by 1000
        return floatval($kg) * 1000;
    }

    public static function toInteger($num): int
    {
        $num = str_replace(',', '.', $num);

        $result = floatval($num);
        if ($result < 1) {
            $result = ceil($result);
        }

        return (int)$result;
    }
}